/* XNS Scientific Notation type */

function SciNotat(num) {
   if (typeof num.xnsType == "undefined") {
      return Number.NaN
      }

   if (arguments.length == 2) {
// literal notation
      var expon = arguments[1]
      var test = ((num.isReal())&&(expon.isReal())&&(expon.isInteger())&&(num.isDecimal()))
      if (test) {
          this.mantissa = num.multiply(1)
          this.exponent = expon.multiply(1)
          this.xnsType = "SciNotat"
          return this
          }
      return Number.NaN
      }

   switch (num.xnsType) {
      case "Number":
// do the conversion for a native number value.

      if (isNaN(num)) {
         return Number.NaN
         }
      var arg = num.multiply(1) // make sure num is a number type and not a string
      if (arg.isNegative()) {
         arg.negate()
         } // make sure num is a non-negative number for logarithm
      var exponent = Math.floor(Math.log(arg) / Math.LN10)
      if (arg == 0) {
         exponent = 0
         } // to fix a bug; ln 0 = -Infinity
      arg /= Math.pow(10, exponent)
      // move the decimal point to right after the first digit

// set properties
      this.mantissa = arg
      this.exponent = exponent
      this.xnsType = "SciNotat"
      if (num.isNegative()) {
         this.mantissa *= -1 // restore negative value
         }
      return this
      break;

	case "BigDecimal":
// do the conversion for a BigDecimal() number value.
	this.exponent = new BigDecimal(0)
	this.mantissa = new BigDecimal(num)
	var sectLength = BigDecimal.addZeroes.length
	if (this.mantissa.decPtAfter > 0) {
		this.exponent.add(new BigDecimal(sectLength * this.decPtAfter))
		this.mantissa.decPtAfter = 0
		}
	sectLength = new BigDecimal(sectLength)
	while (this.mantissa.digitArray[0] * 1 == 0) {
		this.exponent.subtract(sectLength)
		this.mantissa.digitArray.shift()
		}
	var a = 1
	this.mantissa.digitArray[this.mantissa.digitArray.length] = 0
	for (var k = 0; k < this.mantissa.digitArray.length - 1; k++) {
		var j = this.mantissa.digitArray[k].toString().substr(a)
		if (k == 0) {
			this.exponent.add(j.length)
			}
		this.mantissa.digitArray[k] = this.mantissa.digitArray[k].toString().substr(0,a)
		this.mantissa.digitArray[k + 1] = j + this.mantissa.digitArray[k + 1]
		a = BigDecimal.addZeroes.length
		}
	return this
	break;

      case "SciNotat":
// return a clone of the argument.
      this.mantissa = num.mantissa.multiply(1)
      this.exponent = num.exponent.multiply(1)
      this.xnsType = "SciNotat"
      return this
      break;

      default:
// other type of XNS number.
      if (typeof window[num.xnsType] == "function") {
         return window[num.xnsType]["_toSciNotat"](num)
         }
      break;
      }
   return Number.NaN // to eliminate strict warning
   }

SciNotat.prototype.compactNum = function() {
   // must be sure 1 <= Math.abs(response.mantissa) < 10
   while (ineq(-1, LT, this.mantissa, LT, 0, OR, 0, LT, this.mantissa, LT, 1)) {
      this.mantissa = this.mantissa.multiply(10)
      this.exponent = this.exponent.subtract(1)
      }
   while (ineq(this.mantissa, LTE, -10, OR, 10, LTE, this.mantissa)) {
      this.mantissa = this.mantissa.divide(10)
      this.exponent = this.exponent.add(1)
      }
   // mantissa and exponent have now been cleaned up.
   }

SciNotat.prototype.toString = function() {
   var response = "(" + this.mantissa + "e"
   if (this.exponent >= 0) {
      response += "+"
      }
   response += this.exponent + ")"
   return response
   }

SciNotat.prototype.isReal = returnTrue
SciNotat.prototype.isNegative = function() {
   return this.mantissa.isNegative()
   }
SciNotat.prototype.isInteger = function() {
   var b = this.exponent.toString() * 1
   var a = this.mantissa.multiply(Math.pow(10, b))
   return a.isInteger()
   }
SciNotat.prototype.isDecimal = returnTrue
SciNotat.prototype.isRational = returnTrue

SciNotat.prototype.negate = function() {
   var response = new SciNotat(this)
   response.mantissa = response.mantissa.negate()
   return response
   }

SciNotat.prototype.add = function(that) {
   if (typeof that.xnsType == "undefined") {
      return Number.NaN
      }
/* Only XNS numbers have the xnsType property.  Thus, anything not having that property is not 
   an XNS number.  */

   switch (that.xnsType) {
      case "Number":
      case "BigDecimal":
      that = new SciNotat(that)

      case "SciNotat":
      // line up the exponents
      if (!(this.exponent.compareTo(that.exponent) < 2)) {
         return Number.NaN
         }
	var response = new SciNotat(this)
	while (response.exponent.compareTo(that.exponent) == -1) {
		response.mantissa = response.mantissa.divide(10)
		response.exponent = response.exponent.add(1)
		}
      var a = this.mantissa.multiply(1)
      var b = this.exponent.multiply(1)
      var c = that.mantissa.multiply(1)
      var d = that.exponent.multiply(1)
      while (b.compareTo(d) == -1) { // this.exponent < that.exponent
         a = a.divide(10)
         b = b.add(1)
         }
      while (b.compareTo(d) == 1) { // this.exponent > that.exponent
         c = c.divide(10)
         d = d.add(1)
         }
      return new SciNotat(a.add(c), b)
      break;

      default:
      if (that.isDecimal()) {
         return this.add(window[that.xnsType]["_toSciNotat"](num))
         }
      return (SciNotat["_to" + that.xnsType](this)).add(that)
      }
   return Number.NaN
   }

SciNotat.prototype.subtract = function(that) {
   return this.add(that.negate())
   }

SciNotat.prototype.multiply = function(that) {
   if (typeof that.xnsType == "undefined") {
      return Number.NaN
      }
/* Only XNS numbers have the xnsType property.  Thus, anything not having that property is not 
   an XNS number.  */

   switch (that.xnsType) {
      case "Number":
      case "BigDecimal":
      that = new SciNotat(that)

      case "SciNotat":
      var a = this.mantissa.multiply(that.mantissa)
      var b = this.exponent.add(that.exponent)

      return new SciNotat(a, b)
      break;

      default:
      if (that.isDecimal()) {
         return this.multiply(window[that.xnsType]["_toSciNotat"](num))
         }
      return (SciNotat["_to" + that.xnsType](this)).multiply(that)
      }
   return Number.NaN
   }

SciNotat.prototype.divide = function(that) {
   if (typeof that.xnsType == "undefined") {
      return Number.NaN
      }
/* Only XNS numbers have the xnsType property.  Thus, anything not having that property is not 
   an XNS number.  */

   switch (that.xnsType) {
      case "Number":
      that = new SciNotat(that)

      case "SciNotat":
      var a = this.mantissa.divide(that.mantissa)
      var b = this.exponent.subtract(that.exponent)

      return SciNotat.compactNum(a, b)
      break;

      default:
      if (that.isDecimal()) {
         return this.divide(window[that.xnsType]["_toSciNotat"](num))
         }
      return (SciNotat["_to" + that.xnsType](this)).divide(that)
      }
   return Number.NaN
   }

XNS_Types["SciNotat"] = true
