/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Abacus MathML Overlays for Mozilla Composer.
 *
 * The Initial Developer of the Original Code is 
 * Alexander J. Vincent <ajvincent@juno.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

Range.prototype.replaceContents = function(aNewContents) {
  var aRefNode = aNewContents;
  if ((aNewContents.nodeType == 11)&&(aNewContents.lastChild)) {
    aRefNode = aNewContents.lastChild;
  }
  var nodeCountIn = this.startContainer.childNodes.length;
  this.insertNode(aNewContents);
  var nodeCountOut = this.startContainer.childNodes.length;
  if (aRefNode.nextSibling) {
    if (this.collapsed) {
      this.setEndBefore(aRefNode.nextSibling);
    }
    this.setStartBefore(aRefNode.nextSibling);
  }
  var rv = this.extractContents();
  this.deleteContents();
  return rv;
}

const abacusComposerTxn = {
  localAppDoTxn: function() {
    var aDoc = this.mathNode.ownerDocument;
    var links = aDoc.getElementsByTagName("link");
    var linkTest = false;

    this.mathCSS_URL = "chrome://abacus/content/tools/math.css";
    warn("abacusComposerTxn.localAppDoTxn assumes only Abacus-enabled applications will read the edited document.");

    for (var i = 0; i < links.length; i++) {
      var currentLink = links[i];
      if (currentLink.getAttribute("TYPE") != "text/css") {
        continue;
      }
      if (currentLink.getAttribute("REL") != "stylesheet") {
        continue;
      }
      if (currentLink.getAttribute("HREF") != this.mathCSS_URL) {
        continue;
      }
      linkTest = true;
      break;
    }
    if (!linkTest) {
      currentLink = aDoc.createElement("LINK");
      currentLink.setAttribute("TYPE", "text/css");
      currentLink.setAttribute("REL", "stylesheet");
      currentLink.setAttribute("HREF", this.mathCSS_URL);
      aDoc.getElementsByTagName("HEAD")[0].appendChild(currentLink);
    }
  },

  localAppUndoTxn: function() {
    var aDoc = this.mathNode.ownerDocument;
    var links = aDoc.getElementsByTagName("link");

    this.mathCSS_URL = "chrome://abacus/content/tools/math.css";

    for (var i = 0; i < links.length; i++) {
      var currentLink = links[i];
      if (currentLink.getAttribute("TYPE") != "text/css") {
        continue;
      }
      if (currentLink.getAttribute("REL") != "stylesheet") {
        continue;
      }
      if (currentLink.getAttribute("HREF") != this.mathCSS_URL) {
        continue;
      }
      currentLink.parentNode.removeChild(currentLink);
      break;
    }
  }
}

function abacusOpenMathMLEditor() {
  ecmaDebug = true;
  var originalContents;
  var mathNode;
  var idPrefix;
  var parentNode;
  var refNode;
  var editor = GetCurrentEditor();
  assert(editor != null, "We were unable to get an editor!", true);
  var range = editor.selection.getRangeAt(0).cloneRange();
  var docFrag = range.cloneContents();

  var foo = [];
  for (var i = 0; i < docFrag.childNodes.length; i++) {
    foo.push(docFrag.childNodes[i].nodeName);
  }

  switch (docFrag.childNodes.length) {
    case 0:
      parentNode = range.commonAncestorContainer;
      break;

    case 1:
      parentNode = range.commonAncestorContainer;
  }

  if ((docFrag.childNodes.length == 1)&&(docFrag.firstChild.nodeType == 1)&&(docFrag.firstChild.localName == "math")) {
    // we selected a math node
    originalContents = range.startContainer;
    mathNode = docFrag.firstChild;
    idPrefix = originalContents.getAttribute("id");
  } else {
    originalContents = range;
    mathNode = range.startContainer.ownerDocument.createElement("math");
    idPrefix = "";
  }
  var txn = new abacusTransaction(editor.transactionManager, mathNode, originalContents, idPrefix, parentNode);
  txn.acceptMethods(abacusComposerTxn);
  txn.openDialog({});
}

function updateMathMLEditorDisabled() {
  if (!abacusMathMLEditorCmd) {
    abacusMathMLEditorCmd = document.getElementById("abacusOpenMathMLEditor");
  }
  var disabled = true;
  var editor = GetCurrentEditor();
  switch (gEditorDisplayMode) {
    case kDisplayModeSource:
      break; // it's deliberate here

    case kDisplayModeNormal:
    case kDisplayModeAllTags:
    case kDisplayModePreview:
      if (editor.selection.rangeCount > 1) {
        break;  // we may reach this for <table> being edited in Mozilla
      }

      var range = editor.selection.getRangeAt(0);
      if (!range.collapsed) {
        break; // this will change when Bugzilla.m.o bug 253609 is fixed
      }

      disabled = false;
  }

  if (disabled) {
    abacusMathMLEditorCmd.setAttribute("disabled", "true");
  } else {
    abacusMathMLEditorCmd.removeAttribute("disabled");
  }
}

var abacusMathMLEditorCmd;
window.addEventListener("load", function() {
  setInterval(updateMathMLEditorDisabled, 300);
}, true);
