/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Abacus Base Template Editor.
 *
 * The Initial Developer of the Original Code is 
 * Alexander J. Vincent <ajvincent@juno.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function doAccept() {
  switch (aTemplate.templateEditCode) {
    case 0: 
      aTemplate.templateEditCode = 1;
    case 1:
    case 3:
      break;
    case 4:
      aTemplate.templateEditCode = 3;
      break;
    default:
      throw new Error("An unauthorized templateEditCode (" + aTemplate.templateEditCode + ") at doAccept!");
  }
  aTemplate.basePrefix = basePrefix.value;
  aTemplate.wrapperPrefix = wrapperPrefix.value;
  opener.editor.watchUserErrors('templateUpdated', aTemplate);
  return true;
}

function doCancel() {
  opener.editor.watchUserErrors('templateCanceled', aTemplate);
  return true;
}

function correctPostfix(aPrefixId, aPostfixId) {
  var aPrefixNode   = window[aPrefixId];
  var aPostfixNode  = window[aPostfixId];
  var aPrefixValue  = aPrefixNode.value;
  var aPostfixValue = "";
  var invalid = false;

  if (aPrefixValue.charAt(0) != "<") {
    invalid = true;
  }

  if (aPrefixValue.indexOf(">") != aPrefixValue.lastIndexOf(">")) {
    invalid = true;
  }

  if (aPrefixValue.indexOf(">") == -1) {
    invalid = true;
  }

  if (aPrefixValue.lastIndexOf("/") == aPrefixValue.lastIndexOf(">") - 1) {
    invalid = true;
  }

  if (aPrefixValue.lastIndexOf(">") != aPrefixValue.length - 1) {
    invalid = true;
  }

  if (invalid) {
    aPostfixNode.value = "";
  } else {
    var re= /[\s\>]/;
    var tagNameEnds = aPrefixValue.search(re);
    var rootElement = aPrefixValue.substring(1, tagNameEnds);
    aPostfixValue = "</" + rootElement + ">";
    var testDocString = '<!DOCTYPE ' + rootElement + " [\n";
    testDocString += '<!ENTITY % namespaces.dtd SYSTEM "chrome://abacus/content/tools/namespaces.dtd"> %namespaces.dtd;';
    testDocString += '\n]>\n';
    testDocString += aPrefixValue + aPostfixValue;
    var testDoc = parser.parseFromString(testDocString, "application/xml");
    invalid = (parser.checkRootElement(testDoc, "parsererror"));
  }

  if (invalid) {
    aPrefixNode.setAttribute("invalid", "true");
    aPostfixNode.value = "";
  } else {
    aPrefixNode.removeAttribute("invalid");
    aPostfixNode.value = aPostfixValue;
  }

  var acceptDisabled = Boolean(document.getElementsByAttribute("invalid", "true").length);
  if (acceptDisabled) {
    document.documentElement.setAttribute("acceptDisabled", acceptDisabled.toString());
  } else {
    document.documentElement.removeAttribute("acceptDisabled");
  }
  var acceptButton = document.documentElement.getButton("accept");
  var acceptButtonDisabled = acceptButton.hasAttribute("disabled");
  var errMsg = "<xul:dialog/> does not yet support the acceptDisabled attribute.";
  if (!assert(acceptDisabled == acceptButtonDisabled, errMsg)) {
    if (acceptDisabled) {
      acceptButton.setAttribute("disabled", "true");
    } else {
      acceptButton.removeAttribute("disabled");
    }
  }

  return !invalid;
}

ecmaDebug = opener.ecmaDebug;
const parser = opener.parser;
const aTemplate = arguments[0];
assert(aTemplate.templateEditCode != 2, "Template edit code 2 is prohibited in templateDialog.xul!", true);
assert(aTemplate.templateEditCode != 5, "Template edit code 5 is prohibited in templateDialog.xul!", true);
var basePrefix;
var basePostfix;
var wrapperPrefix;
var wrapperPostfix;
var userErrorBox;
window.title = aTemplate.dialogTitle;

window.addEventListener("load", function() {
  userErrorBox   = document.getElementById("usererrorbox");
  basePrefix     = document.getElementById("basePrefix");
  basePostfix    = document.getElementById("basePostfix");
  wrapperPrefix  = document.getElementById("wrapperPrefix");
  wrapperPostfix = document.getElementById("wrapperPostfix");
  basePrefix.value = aTemplate.basePrefix;
  wrapperPrefix.value = aTemplate.wrapperPrefix;
  correctPostfix("basePrefix", "basePostfix");
  correctPostfix("wrapperPrefix", "wrapperPostfix");
}, true);
