/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Abacus Template Editor User Interface Commands.
 *
 * The Initial Developer of the Original Code is 
 * Alexander J. Vincent <ajvincent@juno.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const uiCommands = {
  labelBox: null,
  imageBox: null,
  displayedMenu: null,
  commandSet: null,
  lastPopupShown: null,
  menubarDeck: null,
  uiGroupPopup: null,
  uiControlPopup: null,

  createMenuItem: function(aUIControlNode, aLanguageSet) {
    /* In the actual MathML editor, we would create a <xul:menuitem/> element.
       In the template editor, though, we create a special <xul:menu/> element and append a clone of uiControlPopup.
    */
    var rv = document.createElementNS(XUL_NS, "menu");

    rv.uiNode = aUIControlNode;
    rv.setAttribute("label", aUIControlNode.getAttribute("label"));
    rv.setAttribute("image", aUIControlNode.getAttribute("image"));

    // give the user-interface node a dialog title
    // aLanguageSet is a human language, followed by a slash, followed by a markup language.  Example: en-US/MathML
    var operationName  = aUIControlNode.getAttribute("operationName");
    var indexCode      = aUIControlNode.getAttribute("indexCode");
    var alternateIndex = aUIControlNode.getAttribute("alternateIndex");
    var joinArray = [operationName, indexCode, aLanguageSet, alternateIndex];
    aUIControlNode.dialogTitle = joinArray.join("/");
    // this gives us our standard operationName/indexCode/xml:lang/markupLanguage/alternateIndex format;

    var baseMenuPopup = this.uiControlPopup.cloneNode(true);
    baseMenuPopup.removeAttribute("id");
    rv.appendChild(baseMenuPopup);

    return rv;
  },

  populateMenu: function(aMenu, aUIGroupNode, aLanguageSet) {
    aMenu.setAttribute("label", aUIGroupNode.getAttribute("label"));
    aMenu.setAttribute("image", aUIGroupNode.getAttribute("image"));
    var baseMenuPopup = this.uiGroupPopup.cloneNode(true);
    var aMenuPopup = document.createElementNS(XUL_NS, "menupopup");
    for (var i = 0; i < aUIGroupNode.childNodes.length; i++) {
      var aUIControlNode = aUIGroupNode.childNodes[i];
      var aMenuItem = this.createMenuItem(aUIControlNode, aLanguageSet);
      assert(aMenuItem.uiNode, "aMenuItem did not have a uiNode property!", true);
      aMenuPopup.appendChild(aMenuItem);
    }
    while (baseMenuPopup.firstChild) {
      aMenuPopup.appendChild(baseMenuPopup.firstChild);
    }
    aMenuPopup.setAttribute("onpopupshowing", baseMenuPopup.getAttribute("onpopupshowing"));
    aMenu.appendChild(aMenuPopup);
    aMenu.uiNode = aUIGroupNode;
  },

  populateMenubar: function(aMenubar, aUIGroupNodeArray, aLanguageSet) {
    /*
      A little explanation is in order.

      The aMenubar argument is a menubar which this function must add complete menus to, including menupopups and menuitems.

      The aUIGroupNodeArray is an array of <uigroup/> elements specific to the language of the menubar in question.  The
      idea is this particular array corresponds to a specific xml:lang attribute value; everything is in one human
      language.  Thus, the user will see a menubar of templates specific to his or her own language, in theory.
    */
    for (var i = 0; i < aUIGroupNodeArray.length; i++) {
      var aUIGroupNode = aUIGroupNodeArray[i];
      var aMenu = document.createElementNS(XUL_NS, "menu");
      this.populateMenu(aMenu, aUIGroupNode, aLanguageSet);
      aMenubar.appendChild(aMenu);
      if (typeof aMenubar.uiNode == "undefined") {
        aMenubar.uiNode = aMenu.uiNode.parentNode;
      }
    }
    aMenubar.txmgr = Components.classes["@mozilla.org/transactionmanager;1"].createInstance(nsITransactionManager);
  },
  
  init: function(aUIArray) {
    // identify crucial elements as properties
    this.labelBox = document.getElementById('labelBox');
    this.labelBox.addEventListener("input", function() {
      uiCommands.labelBox.removeAttribute("style");
    }, true);
    this.imageBox = document.getElementById('imageBox');
    this.displayedMenu = document.getElementById('displayedMenu');
    this.uiControlPopup = document.getElementById('uiControlPopup');
    this.uiGroupPopup = document.getElementById('uiGroupPopup');
    this.ui_xmlLang = document.getElementById('ui_xml:lang');
    this.ui_markupLanguage = document.getElementById('ui_markupLanguage');

    // prepare command set
    var commandSet = document.getElementById('uiCommandSet');
    this.commandsArray = [];
    for (var i = 0; i < commandSet.childNodes.length; i++) {
      this.commandsArray.push(commandSet.childNodes[i]);
    }

    // prepare menubar deck
    this.menubarDeck = document.createElementNS(XUL_NS, 'menudeck');
    this.menubarDeck.setAttribute("orient", "vertical");
    for (var title in aUIArray) {
      /* note: title includes both the human language and the markup language.  
         It also has the filename in parentheses.  For example:  en-US/MathML (math.xml)
      */
      var menubar = this.displayedMenu.cloneNode(true);
      menubar.removeAttribute('id');
      menubar.setAttribute('menulabel', title);
      var lang = title.substr(0, title.indexOf(" "));
      var aFilename = title.substring(title.indexOf("(") + 1, title.length - 1);
      menubar.templateFile = editor.templatesTable.getTemplateFileByName(aFilename);
      menubar.aOperationPrefix = aFilename.substr(0, aFilename.indexOf("."));
      if (menubar.aOperationPrefix == "math") {
        menubar.aOperationPrefix = "";
      }
      this.populateMenubar(menubar, aUIArray[title], lang);
      this.menubarDeck.appendChild(menubar);
    }
    this.menubarDeck.appendChild(this.displayedMenu.nextSibling);
    document.getElementById("menubarDeckHolder").appendChild(this.menubarDeck); // this constructs the menudeck

    // fill the ui xml:lang and markupLanguage fields
    this.ui_xmlLang.appendChild(editor.grid.menus["xml:lang"].firstChild.cloneNode(true));
    this.ui_markupLanguage.appendChild(editor.grid.menus["markupLanguage"].firstChild.cloneNode(true));

    this.updateCommandsDisabled(null);
  },

  updateCommandsDisabled: function(aTargetNode) {
    for (var i = 0; i < this.commandsArray.length; i++) {
      var aCommand = this.commandsArray[i];
      var disabled = (eval(aCommand.id)).isCommandDisabled(aTargetNode);
      if (disabled) {
        aCommand.setAttribute("disabled", true);
      } else {
        aCommand.removeAttribute("disabled");
      }
    }
    this.lastPopupShown = aTargetNode;
  },

  runTransaction: function(aCommand) {
    assert(this.lastPopupShown != null, "lastPopupShown == null for " + aCommand.id, true);
    //                 menupopup      menu       menubar
    var menubar = this.lastPopupShown.parentNode.parentNode;
    while (menubar.localName != "menubar") {
      menubar = menubar.parentNode;
    }
    var aTransaction = constructNew(window[aCommand.id]);
    menubar.txmgr.doTransaction(aTransaction);
    this.updateCommandsDisabled(null);
  },

  undoTransaction: function() {
    assert(this.lastPopupShown != null, "lastPopupShown == null for undoTransaction()", true);
    var menubar = this.lastPopupShown.parentNode.parentNode;
    assert(menubar.localName == "menubar", "Expected menubar, received " + menubar.localName, true);
    menubar.txmgr.undoTransaction();
  },

  redoTransaction: function() {
    assert(this.lastPopupShown != null, "lastPopupShown == null for redoTransaction()", true);
    var menubar = this.lastPopupShown.parentNode.parentNode;
    assert(menubar.localName == "menubar", "Expected menubar, received " + menubar.localName, true);
    menubar.txmgr.redoTransaction();
  },

  saveContent: function() {
    var serializer = new XMLSerializer();
    var dummyRootNode = document.createElementNS(MEDIT_NS + "#templates", "foo");
    dummyRootNode.setAttributeNS(XMLNS_NS, "xmlns:xml", XML_NS);
    dummyRootNode.setAttributeNS(XMLNS_NS, "xmlns", MEDIT_NS + "#templates");
    // this is used to fool the serializer into giving us non-prefixed markup

    var deckChildLength = this.menubarDeck.childNodes.length - 1;
    // this.menubarDeck.lastChild.localName == "hbox";
    for (var i = 0; i < deckChildLength; i++) {
      var aMenubar = this.menubarDeck.childNodes[i];
      if ((aMenubar.txmgr.numberOfUndoItems > 0) && (!aMenubar.templateFile.needsSave)) {
        /* We are sensitive to aMenubar.templateFile.needsSave for efficiency reasons.
           After all, once we serialize the userInterfaces node, we really don't need (or want!) to go through this
           process again.
        */

        // serialize menubar.parentNode
        var dummyNodeCopy = dummyRootNode.cloneNode(true);
        dummyNodeCopy.appendChild(aMenubar.uiNode.cloneNode(true));
        var sourceAsText = serializer.serializeToString(dummyNodeCopy, "application/xml");
        var fullSource = sourceAsText.substring(sourceAsText.indexOf(">") + 1, sourceAsText.lastIndexOf("<"));
        fullSource = fullSource.replace(/\>\</gm, ">\n<");
        fullSource = fullSource.replace(/\<uiGroup\s/gm,     "  <uiGroup ");
        fullSource = fullSource.replace(/\<uiControl\s/gm,   "    <uiControl ");
        fullSource = fullSource.replace(/\<\/uiControl\s/gm, "    </uiControl ");
        fullSource = fullSource.replace(/\<\/uiGroup\s/gm,   "  </uiGroup ");

        // replace content in aMenubar.templateFile
        var originalSource = aMenubar.uiNode.sourceObject.fullElement;
        var stringParts = [];

        // find insertion point and cut length for template
        var startIndex = aMenubar.templateFile.fileAsString.indexOf(originalSource);
        var cutLength;
        if (startIndex == -1) {
          cutLength = 0; // we are not cutting any characters from the template file
          startIndex = aMenubar.templateFile.fileAsString.indexOf("</userInterfaces>");
        } else {
          cutLength = originalSource.length;
        }

        stringParts[0] = aMenubar.templateFile.fileAsString.substr(0, startIndex);
        stringParts[1] = fullSource;
        stringParts[2] = aMenubar.templateFile.fileAsString.substr(startIndex + cutLength);

        aMenubar.templateFile.fileAsString = stringParts.join("");
        aMenubar.uiNode.sourceObject.fullElement = fullSource; // avoid trouble later from a future save
        aMenubar.templateFile.needsSave = true;
      }
    }
  },

  toString: function() {
    return "[object uiCommands]";
  }
}

const uiUndo = {
  isCommandDisabled: function(aPopupNode) {
    var rv = true;
    if (!aPopupNode) {
      return rv;
    }
    var aNode = aPopupNode.parentNode.parentNode;
    if (aNode.nodeName == "menubar") {
      rv = (aNode.txmgr.numberOfUndoItems == 0);
    }
    return rv;
  }
}

const uiRedo = {
  isCommandDisabled: function(aPopupNode) {
    var rv = true;
    if (!aPopupNode) {
      return rv;
    }
    var aNode = aPopupNode.parentNode.parentNode;
    if (aNode.nodeName == "menubar") {
      rv = (aNode.txmgr.numberOfRedoItems == 0);
    }
    return rv;
  }
}

function uiInsertTransaction() {
}
uiInsertTransaction.prototype = new nsITransaction();
uiInsertTransaction.prototype.undoTransaction = function() {
  this.insertedUINode.parentNode.removeChild(this.insertedUINode);
  this.insertedNode.parentNode.removeChild(this.insertedNode);
}

function uiAddNewMenu() {
  this.insertedNode = document.createElementNS(XUL_NS, "menu");
  this.insertedNode.setAttribute("label", uiCommands.labelBox.value);
  this.insertedNode.setAttribute("image", uiCommands.imageBox.value);
  var menupopup = document.getElementById("uiGroupPopup").cloneNode(true);
  menupopup.removeAttribute("id");
  this.insertedNode.appendChild(menupopup);

  this.insertedUINode = document.createElementNS(MEDIT_NS + "#templates", "uiGroup");
  this.insertedUINode.setAttribute("label", uiCommands.labelBox.value);
  this.insertedUINode.setAttribute("image", uiCommands.imageBox.value);
  this.insertedNode.uiNode = this.insertedUINode;
}
uiAddNewMenu.prototype = new uiInsertTransaction();
uiAddNewMenu.prototype.doTransaction = function() {
  var aMenubar = uiCommands.menubarDeck.childNodes[uiCommands.menubarDeck.selectedIndex];
  aMenubar.appendChild(this.insertedNode);
  aMenubar.uiNode.appendChild(this.insertedUINode);
}
uiAddNewMenu.isCommandDisabled = function(aNode) {
  return false; // we can always add a new menu
}

function uiAddNewItem() {
  this.insertedNode = document.createElementNS(XUL_NS, "menu");
  this.insertedNode.setAttribute("label", uiCommands.labelBox.value);
  this.insertedNode.setAttribute("image", uiCommands.imageBox.value);
  var menupopup = document.getElementById("uiControlPopup").cloneNode(true);
  menupopup.removeAttribute("id");
  this.insertedNode.appendChild(menupopup);

  this.insertedUINode = document.createElementNS(MEDIT_NS + "#templates", "uiControl");
  this.insertedUINode.setAttribute("label", uiCommands.labelBox.value);
  this.insertedUINode.setAttribute("image", uiCommands.imageBox.value);
  this.insertedUINode.dialogTitle = ""; // used later in uiUpdateTemplate;
  this.insertedNode.uiNode = this.insertedUINode;

  this.popup = uiCommands.lastPopupShown;
  var aNode = this.popup.parentNode;
  assert(aNode.nodeName == "menu", "Expected menu, found " + aNode.nodeName, true);
  if (aNode.uiNode.localName == "uiControl") {
    this.popup = aNode.parentNode;
    aNode = this.popup.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  for (var i = 0; i < this.popup.childNodes.length; i++) {
    var sepNode = this.popup.childNodes[i];
    if (sepNode.nodeName == "menuseparator") {
      this.separator = sepNode;
      break;
    }
  }
  this.uiNode = this.popup.parentNode.uiNode;
}
uiAddNewItem.prototype = new uiInsertTransaction();
uiAddNewItem.prototype.doTransaction = function() {
  this.popup.insertBefore(this.insertedNode, this.separator);
  this.uiNode.appendChild(this.insertedUINode);
}
uiAddNewItem.isCommandDisabled = function(aNode) {
  return false; // we can always add a new item;
}

function uiMoveMenuLeft() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;
}
uiMoveMenuLeft.prototype = new nsITransaction();
uiMoveMenuLeft.prototype.doTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu, this.menu.previousSibling);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode, this.menu.uiNode.previousSibling);
}
uiMoveMenuLeft.prototype.undoTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu.nextSibling, this.menu);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode.nextSibling, this.menu.uiNode);
}
uiMoveMenuLeft.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  var node1 = aNode;
  assert(aNode.uiNode, aNode.nodeName, true);
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  return (aNode.previousSibling == uiCommands.displayedMenu.firstChild);
}

function uiMoveMenuRight() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;
}
uiMoveMenuRight.prototype = new nsITransaction();
uiMoveMenuRight.prototype.doTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu.nextSibling, this.menu);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode.nextSibling, this.menu.uiNode);
}
uiMoveMenuRight.prototype.undoTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu, this.menu.previousSibling);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode, this.menu.uiNode.previousSibling);
}
uiMoveMenuRight.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  return (aNode == uiCommands.displayedMenu.lastChild);
}

function uiDeleteMenu() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;
  this.parentNode = aNode.parentNode;
  this.nextSibling = aNode.nextSibling;
  this.nextUINode = (this.nextSibling) && (typeof this.nextSibling.uiNode != "undefined") ? this.nextSibling.uiNode : null;
  this.uiParentNode = aNode.uiNode.parentNode;
}
uiDeleteMenu.prototype = new nsITransaction();
uiDeleteMenu.prototype.doTransaction = function() {
  this.menu.parentNode.removeChild(this.menu);
  this.menu.uiNode.parentNode.removeChild(this.menu.uiNode);
}
uiDeleteMenu.prototype.undoTransaction = function() {
  if (this.nextSibling) {
    this.parentNode.insertBefore(this.menu, this.nextSibling);
  } else {
    this.parentNode.appendChild(this.menu);
  }

  if (this.nextUINode) {
    this.uiParentNode.insertBefore(this.menu.uiNode, this.nextUINode);
  } else {
    this.uiParentNode.appendChild(this.menu.uiNode);
  }
}
uiDeleteMenu.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  return false; // once we find a menu to delete, we know we can delete it.
}

function uiUpdateMenu() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;

  this.label = uiCommands.labelBox.value;
  this.previousLabel = aNode.uiNode.getAttribute("label");
//this.imageSrc = uiCommands.imageBox.value;
//this.previousImageSrc = aNode.uiNode.getAttribute("image");
}
uiUpdateMenu.prototype = new nsITransaction();
uiUpdateMenu.prototype.doTransaction = function() {
  this.menu.uiNode.setAttribute("label", this.label);
  this.menu.setAttribute("label", this.label);
//this.menu.uiNode.setAttribute("image", this.imageSrc);
//this.menu.setAttribute("image", this.imageSrc);
  uiCommands.labelBox.setAttribute("style", "background-color: #7fffff;"); // visual confirmation for user
}
uiUpdateMenu.prototype.undoTransaction = function() {
  this.menu.uiNode.setAttribute("label", this.previousLabel);
  this.menu.setAttribute("label", this.previousLabel);
//this.menu.uiNode.setAttribute("image", this.previousImageSrc);
//this.menu.setAttribute("image", this.previousImageSrc);
  uiCommands.labelBox.removeAttribute("style");
}
uiUpdateMenu.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName == "uiControl") {
    aNode = aNode.parentNode.parentNode;
  }
  assert(aNode.uiNode.localName == "uiGroup", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  return false; // once we find a control to update, we know we can update it.
}

function uiMoveItemUp() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  assert(aNode.uiNode.localName == "uiControl", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;
}
uiMoveItemUp.prototype = new nsITransaction();
uiMoveItemUp.prototype.doTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu, this.menu.previousSibling);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode, this.menu.uiNode.previousSibling);
}
uiMoveItemUp.prototype.undoTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu.nextSibling, this.menu);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode.nextSibling, this.menu.uiNode);
}
uiMoveItemUp.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName != "uiControl") {
    return true; // we landed on a uiGroup, and this command is disabled
  }

  return (!aNode.previousSibling);
}

function uiMoveItemDown() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  assert(aNode.uiNode.localName == "uiControl", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;
}
uiMoveItemDown.prototype = new nsITransaction();
uiMoveItemDown.prototype.doTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu.nextSibling, this.menu);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode.nextSibling, this.menu.uiNode);
}
uiMoveItemDown.prototype.undoTransaction = function() {
  this.menu.parentNode.insertBefore(this.menu, this.menu.previousSibling);
  this.menu.uiNode.parentNode.insertBefore(this.menu.uiNode, this.menu.uiNode.previousSibling);
}
uiMoveItemDown.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName != "uiControl") {
    return true; // we landed on a uiGroup, and this command is disabled
  }
  return (aNode.nextSibling.localName == "menuseparator");
}

function uiDeleteItem() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  assert(aNode.uiNode.localName == "uiControl", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;
  this.parentNode = aNode.parentNode;
  this.nextSibling = aNode.nextSibling;
  this.nextUINode = (this.nextSibling) && (typeof this.nextSibling.uiNode != "undefined") ? this.nextSibling.uiNode : null;
  this.uiParentNode = aNode.uiNode.parentNode;
}
uiDeleteItem.prototype = new nsITransaction();
uiDeleteItem.prototype.doTransaction = function() {
  this.menu.parentNode.removeChild(this.menu);
  this.menu.uiNode.parentNode.removeChild(this.menu.uiNode);
}
uiDeleteItem.prototype.undoTransaction = function() {
  if (this.nextSibling) {
    this.parentNode.insertBefore(this.menu, this.nextSibling);
  } else {
    this.parentNode.appendChild(this.menu);
  }

  if (this.nextUINode) {
    this.uiParentNode.insertBefore(this.menu.uiNode, this.nextUINode);
  } else {
    this.uiParentNode.appendChild(this.menu.uiNode);
  }
}
uiDeleteItem.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName != "uiControl") {
    return true; // we landed on a uiGroup, and this command is disabled
  }
  return false; // once we find a control to delete, we know we can delete it.
}

function uiUpdateItem() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  assert(aNode.uiNode.localName == "uiControl", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  this.menu = aNode;

  this.label = uiCommands.labelBox.value;
  this.previousLabel = aNode.uiNode.getAttribute("label");
//this.imageSrc = uiCommands.imageBox.value;
//this.previousImageSrc = aNode.uiNode.getAttribute("image");
}
uiUpdateItem.prototype = new nsITransaction();
uiUpdateItem.prototype.doTransaction = function() {
  this.menu.uiNode.setAttribute("label", this.label);
  this.menu.setAttribute("label", this.label);
//this.menu.uiNode.setAttribute("image", this.imageSrc);
//this.menu.setAttribute("image", this.imageSrc);
  uiCommands.labelBox.setAttribute("style", "background-color: #7fffff;"); // visual confirmation for user
}
uiUpdateItem.prototype.undoTransaction = function() {
  this.menu.uiNode.setAttribute("label", this.previousLabel);
  this.menu.setAttribute("label", this.previousLabel);
//this.menu.uiNode.setAttribute("image", this.previousImageSrc);
//this.menu.setAttribute("image", this.previousImageSrc);
  uiCommands.labelBox.removeAttribute("style");
}
uiUpdateItem.isCommandDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName != "uiControl") {
    return true; // we landed on a uiGroup, and this command is disabled
  }
  return false; // once we find a control to update, we know we can update it.
}

function uiUpdateTemplate() {
  var aNode = uiCommands.lastPopupShown.parentNode;
  assert(aNode.uiNode.localName == "uiControl", "We arrived at the wrong aNode (" + aNode.nodeName + ")!", true);
  assert(aNode.uiNode.parentNode, "A uiControl node is an orphan!", true);
  var parentName = aNode.uiNode.parentNode.localName
  assert(parentName == "uiGroup", "A uiControl node has an invalid parent (" + parentName + ")", true);
  this.previousControl = aNode.uiNode;
  this.previousDialogTitle = this.previousControl.dialogTitle;

  var menuset = editor.grid.menus;
  var dialogTitle = menuset["operationName"].value + "/";
  dialogTitle    += menuset["indexCode"].value + "/";
  dialogTitle    += menuset["xml:lang"].value + "/";
  dialogTitle    += menuset["markupLanguage"].value + "/";
  dialogTitle    += menuset["alternateIndex"].value;
  /*
    This effectively excludes all content templates.  Content templates never have 
    more than one slash in their dialogTitle.  Thus, the editor will only find a template
    if (a) the user has designated a presentation template, and (b) the template is in the
    table.
  */
  this.newDialogTitle = dialogTitle;

  this.newControl = this.previousControl.cloneNode(true);
  this.newControl.setAttribute("operationName",      menuset["operationName"].value);
  this.newControl.setAttribute("indexCode",          menuset["indexCode"].value);
  // we do not set xml:lang or markupLanguage on uiControl nodes; that's managed at the uiGroups element.
  this.newControl.setAttribute("alternateIndex",     menuset["alternateIndex"].value);
  this.newControl.dialogTitle = dialogTitle;

  this.masterNode = aNode;
}
uiUpdateTemplate.prototype = new nsITransaction();
uiUpdateTemplate.prototype.doTransaction = function() {
  this.previousControl.parentNode.replaceChild(this.newControl, this.previousControl);
  this.masterNode.uiNode = this.newControl;
}
uiUpdateTemplate.prototype.undoTransaction = function() {
  this.newControl.parentNode.replaceChild(this.previousControl, this.newControl);
  this.masterNode.uiNode = this.previousControl;
}
uiUpdateTemplate.isCommandReallyDisabled = function(aPopupNode) {
  if (!aPopupNode) {
    return true;
  }
  var aNode = aPopupNode.parentNode;
  if (typeof aNode.uiNode == "undefined") {
    return true;
  }
  if (aNode.uiNode.localName != "uiControl") {
    return true; // we landed on a uiGroup, and this command is disabled
  }

  // we got this far, which means we're on a uiControl.
  var menuset = editor.grid.menus;

  // are we in a menuset that allows the requested xml:lang and markupLanguage?
  var aLanguageSet = menuset["xml:lang"].value + "/" + menuset["markupLanguage"].value;
  var uiGroupsNode = aNode.uiNode.parentNode.parentNode;
  assert(uiGroupsNode.localName == "uiGroups", uiGroupsNode.nodeName, true);
  var matchLanguageSet = uiGroupsNode.getAttributeNS(XML_NS, "lang") + "/" + uiGroupsNode.getAttribute("markupLanguage");
  if (aLanguageSet != matchLanguageSet) {
    return true; // we are NOT in the right menuset.
  }

  // check against menubar.aOperationPrefix.  If it doesn't match, disable.
  // this is a safety catch to make sure we only set user interfaces for templates in the same file.
  var operationPrefix = menuset["operationName"].value;
  operationPrefix = operationPrefix.substr(0, operationPrefix.indexOf(":"));
  //            menu  menupopup  menu       menubar
  var menubar = aNode.parentNode.parentNode.parentNode;
  assert(menubar.localName == "menubar", "Expected menubar, received" + menubar.localName, true);
  if (operationPrefix != menubar.aOperationPrefix) {
    return true;
  }

  var dialogTitle = menuset["operationName"].value + "/";
  dialogTitle    += menuset["indexCode"].value + "/";
  dialogTitle    += aLanguageSet + "/";
  dialogTitle    += menuset["alternateIndex"].value;
  /*
    This effectively excludes all content templates.  Content templates never have 
    more than one slash in their dialogTitle.  Thus, the editor will only find a template
    if (a) the user has designated a presentation template, and (b) the template is in the
    table.
  */
  var searchCriteria = [{name: "dialogTitle", value: dialogTitle}];
  var notResult = editor.templatesTable.getFirstSearchResult(searchCriteria);
  return !notResult; // a null return from getFirstSearchResult means this should be disabled, so return true for a null.
}
uiUpdateTemplate.isCommandDisabled = function(aPopupNode) {
  // this whole function is a hack to force using dialogTitle as the label of our uiControl popup.
  var rv = this.isCommandReallyDisabled(aPopupNode);
  if (rv) {
    if (!aPopupNode) {
      return true;
    }
    var aNode = aPopupNode.parentNode;
    if (typeof aNode.uiNode == "undefined") {
      return true;
    }
    if (aNode.uiNode.localName != "uiControl") {
      return true; // we landed on a uiGroup, and this command is disabled
    }
  }
  aPopupNode.lastChild.setAttribute("label", aPopupNode.parentNode.uiNode.dialogTitle);
  return rv;
}
