/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Abacus xml:lang Identifier.
 *
 * The Initial Developer of the Original Code is 
 * Alexander J. Vincent <ajvincent@juno.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

if (typeof XML_NS == "undefined") {
  window.XML_NS = "http://www.w3.org/XML/1998/namespace";
}

if (typeof XHTML_NS == "undefined") {
  window.XHTML_NS = "http://www.w3.org/1999/xhtml";
}

function isXMLDocument(aDocument) {
  if ((aDocument instanceof XMLDocument)||(aDocument instanceof XULDocument)) {
    return true;
  }
  if ((aDocument instanceof HTMLDocument)&&(aDocument.contentType != "text/html")) {
    return true;
  }
  return false;
}

const hasLangAttrFilter = {
  acceptNode: function(aNode) {
    if ((this.isXML)&&(aNode.hasAttributeNS(XML_NS, "lang"))) {
      return NodeFilter.FILTER_ACCEPT;
    }
    if ((this.isHTML)&&(aNode.hasAttribute("lang"))) {
      if ((!this.isXML)||(aNode.namespaceURI == XHTML_NS)) {
        return NodeFilter.FILTER_ACCEPT;
      }
    }
    return NodeFilter.FILTER_SKIP;
  },

  isXML: false,
  isHTML: false
};

function getLanguage(aElement) {
  var rv = "";
  var aNode = aElement;
  var aDoc = aElement.ownerDocument;
  var walker = aDoc.createTreeWalker(aDoc.documentElement, NodeFilter.SHOW_ELEMENT, hasLangAttrFilter, true);
  var metaList;
  var metaNode;
  var i;
  walker.currentNode = aNode;
  if (aNode.nodeType != 1) {
    aNode = aNode.parentNode;
    walker.currentNode = aNode;
  }
  assert(aNode, "TreeWalker is seriously broken!", true);
  hasLangAttrFilter.isXML = isXMLDocument(aDoc);
  hasLangAttrFilter.isHTML = (aDoc instanceof HTMLDocument);
  do {
    if (hasLangAttrFilter.isXML) {
      rv = aNode.getAttributeNS(XML_NS, "lang");
    }
    if (!rv && hasLangAttrFilter.isHTML) {
      if (!hasLangAttrFilter.isXML || (aNode.namespaceURI == XHTML_NS)) {
        // we check aNode.namespaceURI in case we have <foo:bar lang="baz"/>
        rv = aNode.getAttribute("lang");
      }
    }
    if (!rv) {
      aNode = walker.parentNode();
    }
  } while (!rv && aNode);

  if (!rv && hasLangAttrFilter.isHTML) {
    // okay, check the meta nodes
    if (hasLangAttrFilter.isXML) {
      metaList = document.getElementsByTagNameNS(XHTML_NS, "meta");
    } else {
      metaList = document.getElementsByTagName("meta");
    }
    for (i = 0; i < metaList.length; i++) {
      metaNode = metaList[i];
      if (metaNode.httpEquiv == "Content-Language") {
        rv = metaNode.content;
      }
    }
  }
  return rv;
}
