/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Abacus MathML Templates Table Constructor.
 *
 * The Initial Developer of the Original Code is 
 * Alexander J. Vincent <ajvincent@juno.com>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function abacusTemplateFile(aURL, aTemplateFileCode) {
  this.url = aURL;
  this.templateFileCode = aTemplateFileCode;
  this.needsSave = false;

  // read file into string
  this.urlPath = fUtils.chromeToURL(this.url);
  this.localPath = fUtils.urlToPath(this.urlPath);
  this.fileObj = new File(this.localPath);
  assert(this.fileObj.exists(), aURL + "  does not exist.", true);
  this.fileObj.open("r");
  this.fileAsString = this.fileObj.read();
  this.fileObj.close();

  // parse file into XML document
  this.fileDoc = parser.parseFromString(this.fileAsString, "application/xml");
  assert(parser.checkRootElement(this.fileDoc, "templates"), aURL + " could not be loaded as an XML document.", true);

  // Whitespace is evil, and included only for human reading.  Scripts should remove it from the right namespaces.
  removeWhitespaceByNamespace(this.fileDoc, MEDIT_NS + "#templates");

  // identify content and presentation templates, per the DTD
  this.contentTemplates = this.fileDoc.getElementsByTagName("contentTemplates")[0];
  this.presentationTemplates = this.fileDoc.getElementsByTagName("presentationTemplates")[0];
  this.userInterfaces = this.fileDoc.getElementsByTagName("userInterfaces")[0];

  // Extract template and uiGroups source code
  this.templateStrings = this.quickXMLParse("template");
  this.uiGroupsStrings = this.quickXMLParse("uiGroups");
}
abacusTemplateFile.prototype = {
  newLineRE: /\r\n/,

  precondition: function(aURL, aTemplateFileCode) {
    assert((typeof JS_LIB_LOADED == 'boolean')&&(JS_LIB_LOADED), "JSLib is not loaded!", true);
  },

  postcondition: function(aURL, aTemplateFileCode, rv) {
    assert(!this.newLineRE.test(this.fileAsString), aURL + " has Windows new line characters (\\r\\n).");
  },

  /* Quick & dirty XML parsing, so we can later compare a node to its source code.
     Although we really, really hope the template file doesn't have something like:
    <template>
      <template xmlns="foo">
      </template>
    </template>

     we are prepared for it.

     This code is probably not the most efficient for the desired task, and could probably use some re-engineering.
  */

  quickXMLParse: function(desiredTagName) {
    var preMarkup = "<foo xmlns:math='" + MATH_NS + "' xmlns:mEdit='" + MEDIT_NS + "'>";
    dump("preMarkup: " + preMarkup + "\n");
    dump("quickXMLParse(" + desiredTagName + ")\n");
    var rv = [];
    var source = this.fileAsString + "";
    var openTagIndex = source.indexOf("<" + desiredTagName +" ");
    while (openTagIndex != -1) {
      var fullElement = "";
      var contentDocTest = false;
      do {
        dump(openTagIndex + "\n");
        var closeTagIndex = source.indexOf("</"+ desiredTagName + ">") + (3 + desiredTagName.length);
        dump("close: " + closeTagIndex + "\n");
        assert(closeTagIndex > openTagIndex, "Bounce!");
        fullElement += source.slice(openTagIndex, closeTagIndex);
        var openTagEnds = fullElement.indexOf(">") + 1;
        var openTag = fullElement.substr(0, openTagEnds);
        var openTagAttrs = openTag.match(/(\w*\x3a)?\w+\s*=([\"\'])[^\2]*?\2/gm);
        source = source.substr(closeTagIndex);

        var content = fullElement.substring(openTagEnds, fullElement.length - (3 + desiredTagName.length));
        var feed = preMarkup + fullElement + "</foo>";
        var contentDoc = parser.parseFromString(feed, "application/xml");
        contentDocTest = (parser.checkRootElement(contentDoc, "foo"));
        // this is a sanity check, in case we did end at the wrong closing tag
        if (contentDocTest) {
          openTagIndex = source.indexOf("<" + desiredTagName +" ");
        }
        dump("well-formed: " + contentDocTest + "\n");
        if (!contentDocTest) {
          dump(fullElement + "\n");
        }
      } while (!contentDocTest);
      content = content.substring(content.indexOf("<"), content.lastIndexOf(">") + 1);
      var sourceObject = {
        attrs: openTagAttrs,
        content: content,
        fullElement: fullElement,
        openTag: openTag
      };
      rv.push(sourceObject);
      openTagIndex = source.indexOf("<" + desiredTagName +" ");
    }
    // end while (openTagIndex != -1)
    return rv;
  },

  /*
    matchNodeToSource: the second half of the quick and dirty XML parsing begun in quickXMLParse().

    As written, this method really doesn't need to go in abacusTemplateFile.prototype. But it's a sensible place for it,
    considering that it would be called for a node in the file it belongs to.

    aStringArray must be a return from abacusTemplateFile.prototype.quickXMLParse().
  */

  matchNodeToSource: function(aNode, aStringArray, aRequiredAttrsByName, aCanAcceptOtherAttrs) {
    var rv = null;
    for (var i = 0; i < aStringArray.length; i++) {
      var currentAttrs = aStringArray[i].attrs;
      var check = true;
      var currentAttrNames = [];
      for (var j = 0; j < currentAttrs.length; j++) {
        // continuing with quick & dirty XML parsing, at the attribute level...
        var singleQuote = currentAttrs[j].indexOf("'");
        var doubleQuote = currentAttrs[j].indexOf('"');
        assert((singleQuote != -1)||(doubleQuote != -1), "Attribute string did not find a quote mark.", true);
        var quoteIndex = (singleQuote != -1) ? singleQuote : doubleQuote;
        var attrValue = currentAttrs[j].substring(quoteIndex + 1, currentAttrs[j].length - 1);
        var attrName = currentAttrs[j].substr(0, currentAttrs[j].search(/[\s=]/gm));

        var foundRequiredAttr = false;
        for (var k = 0; k < aRequiredAttrsByName.length; k++) {
          // ugh, three nested loops in this function... is there any way we can trim this down?
          if (aRequiredAttrsByName[k] == attrName) {
            foundRequiredAttr = true;
            break;
          }
        }
        if (!foundRequiredAttr) {
          // this is an attribute we are not checking for.
          continue;
        }

        var namespaceURI = null;
        var colonIndex = attrName.indexOf(":");
        if (colonIndex != -1) {
          // namespace prefix... get the namespace URI that matches
          var namespacePrefix = attrName.substr(0, colonIndex);
          switch (namespacePrefix) {
            case "xml":
              namespaceURI = XML_NS;
              break;

            default:
              namespaceURI = aNode.lookupNamespaceURI(namespacePrefix);
          }
        }

        if (aNode.getAttributeNS(namespaceURI, attrName.substr(colonIndex + 1)) != attrValue) {
          // we found an attribute that doesn't match.
          check = false;
          break;
        }
        currentAttrNames.push(attrName); // we use this in a moment to make sure we got all the attributes!
      }
      if (!check) {
        // go on to the next set of attributes
        continue;
      }

      currentAttrNames.sort();
      if ((currentAttrNames.length != aRequiredAttrsByName.length)&&(!aCanAcceptOtherAttrs)) {
        // wrong set!
        continue;
      }
      for (j = 0; j < currentAttrNames.length; j++) {
        if (currentAttrNames[j] != aRequiredAttrsByName[j]) {
          // attribute mismatch!
          check = false;
          break;
        }
      }
      if (!check) {
        continue;
      }

      // we have the right set.
      rv = aStringArray[i];
      assert(rv.fullElement, "aStringArray[i] did not have a fullElement property!", true);
      aStringArray.splice(i,1); // make the next search easier by removing the correct set
      break;
    }
    return rv;
  },

  save: function() {
    assert(this.needsSave, this.url + " does not need to be saved."); // this is a warning, not an actual assertion
    assert(this.fileObj.open("w") == JS_LIB_OK, "Could not open " + this.url + " for writing!", true);
    var rv = this.fileObj.write(this.fileAsString);
    assert(rv == JS_LIB_OK, "Could not write source code to " + this.url + "!", true);
    this.fileObj.close();
    return rv;
  },

  save_precondition: function() {
    var parseCheckDoc = parser.parseFromString(this.fileAsString, "application/xml");
    var errMsg = "abacusTemplateFile.prototype.save() did not receive a string of well-formed XML markup!";
    assert(parser.checkRootElement(parseCheckDoc, "templates"), errMsg, true);
  },

  save_postcondition: function(rv) {
    var errMsg = "File contents do not match source string for " + this.url + "!";
    this.fileObj.open("r");
    assert(this.fileAsString == this.fileObj.read(), errMsg, true);
    this.fileObj.close();
  },

  toString: function() {
    return "[object abacusTemplateFile]";
  }
}
abacusTemplateFile.BASE_FILE = 1;
abacusTemplateFile.MATH_FILE = 2;
abacusTemplateFile.UNKNOWN_FILE = -1;
setContract(abacusTemplateFile.prototype.save,
            abacusTemplateFile.prototype.save_precondition,
            abacusTemplateFile.prototype.save_postcondition);

function abacusControl(aControl) {
  this.node    = aControl;
  this.name    = aControl.getAttribute("name");
  this.label   = aControl.getAttribute("label");
  this.min     = aControl.getAttribute("min");
  this.max     = aControl.getAttribute("max");
  this.value   = aControl.getAttribute("value");
  aControl.controlObject = this;
}
abacusControl.prototype = {
  precondition: function(aControl) {
    var condition = ((aControl.localName == "control")&&(aControl.namespaceURI == MEDIT_NS));
    assert(condition, "abacusControl must receive a control element.", true);
  },

  toString: function() {
    return "[object abacusControl]";
  }
}

function abacusTemplate(aTemplateNode, aTemplateFile) {
  this['xml:lang']       = "";
  this.node              = aTemplateNode;
  this.templateFile      = aTemplateFile;
  this.templateSource    = "";
  this.matchingTemplates = [this];
  this.isBase            = false;
  this.baseTemplate      = null;
  this.isNewTemplate     = false; // we set this to true only in editor.js#addTemplateToTable().
  this.basePrefix        = "";
  this.wrapperPrefix     = "";
  this.markupLanguage    = "";
  this.alternateIndex    = "";
  this.templateEditCode  = 2;

  /*
       templateEditCode values:
         0  Template is in templates table and identical to template node in the template file.
         1  Template has changed from its original value, and should be updated in the template file.
         2  Template did not exist before and should be added to the templates table.
         3  Template did not exist before and should be added to the templates table and template file.
         4  Template did not exist before, but is being written before adding to templates table.
         5  Template was canceled.

         The value may go from:
 undefined -> 2 (initial setting for template)
         2 -> 0 (abacusTemplatesTable.prototype.addTemplatesByNode)
         0 -> 1 (in templateDialog.xul, onaccept())
         1 -> 0 (this.saveContent())
         2 -> 4 (when being constructed as a brand-new template, abacusTemplateEditor.prototype.addNewTemplate)
         4 -> 3 (in templateDialog.xul, onaccept())
         3 -> 1 (after adding to templates table, abacusTemplateEditor.prototype.addTemplateToTable)
         4 -> 5 (in templateDialog.xul, oncancel())
         4 -> 5 (abacusTemplateEditor.prototype.templateCanceled)
         
  */

  this.operationName     = this.node.getAttribute("operationName");
  this.indexCode         = this.node.getAttribute("indexCode");
  this.dialogTitle       = this.operationName + "/" + this.indexCode;
  this.isContentTemplate = this.node.parentNode ? (this.node.parentNode.localName == "contentTemplates") : arguments[2];
  if (!this.isContentTemplate) {
    this['xml:lang']     = this.node.getAttributeNS(XML_NS, "lang");
    this.markupLanguage  = this.node.getAttribute("markupLanguage");
    this.alternateIndex  = this.node.getAttribute("alternateIndex");
    this.dialogTitle    += "/" + this['xml:lang'] + "/" + this.markupLanguage + "/" + this.alternateIndex;
  }

  if (this.operationName == "base:base") {
    this.isBase = true;
    // set initial prefix in case we're creating a new template
    this.basePrefix = 'xmlns:mEdit="' + MEDIT_NS + '"\n>';
  }

  if ((this.isBase)&&(aTemplateFile)) {
    // set basePrefix and wrapperPrefix values
    var wrapperPrefixNode = this.node.lastChild;
    assert(wrapperPrefixNode, "Uh oh!  Somehow, wrapperPrefixNode == null", true);
    var basePrefixNode = wrapperPrefixNode.previousSibling;
    assert(basePrefixNode.nodeName == "basePrefix", "basePrefix element for base:base template not found!", true);
    assert(wrapperPrefixNode.nodeName == "wrapperPrefix", "wrapperPrefix element for base:base template not found!", true);
    if (basePrefixNode.firstChild) {
      this.basePrefix    = basePrefixNode.firstChild.nodeValue;
      this.wrapperPrefix = wrapperPrefixNode.firstChild.nodeValue;
    }
  }

  this.contentNode = this.node.firstChild;

  if (aTemplateFile) {
    // identify template string from aTemplateFile;
    var templateSet = aTemplateFile.templateStrings;
    var requiredAttrs = ["operationName", "indexCode"];
    if (!this.isContentTemplate) {
      requiredAttrs = requiredAttrs.concat("xml:lang", "markupLanguage", "alternateIndex");
    }
    this.fileExcerpt = aTemplateFile.matchNodeToSource(this.node, templateSet, requiredAttrs.sort(), false);
    if (this.fileExcerpt) {
      this.templateSource = this.fileExcerpt.content;
    }
  }

  // initialize controls
  this.controls = [];
  if (this.contentNode) {
    var cList = this.contentNode.nextSibling;
    if ((cList)&&(cList.tagName == "controlList")) {
      this.initControls(cList);
    }
  }
}
abacusTemplate.prototype = {
  precondition: function(aTemplateNode, aTemplateFile) {
    assert(aTemplateNode.tagName == 'template', "abacusTemplate must receive a template element.", true);
    assert(aTemplateFile instanceof abacusTemplateFile, "abacusTemplate should receive a template file.", true);
  },

  postcondition: function(aTemplateNode, aTemplateFile, rv) {
    var rvMessage = "postcondition did not receive constructed object as return value!";
    assert(arguments[arguments.length - 1] instanceof abacusTemplate, rvMessage, true);
  },

  initControls: function(aControlList) {
    // no need to build a new TreeWalker just for this
    for (var k = 0; k < aControlList.childNodes.length; k++) {
      var aNode = aControlList.childNodes[k];
      var aControl = applyContractConstructor(abacusControl, aNode);
      this.controls.push(aControl);
    }
  },

  addMatchingTemplate: function(aTemplate) {
    this.matchingTemplates.push(aTemplate);
  },

  saveContent: function() {
    var fullSource;
    var control;
    var controlAttr;
    var i;
    var j;
    var cutLength;
    var startIndex;
    var stringParts = [];
    if (!this.node.parentNode) {
      if (this.isContentTemplate) {
        this.templateFile.contentTemplates.appendChild(this.node);
      } else {
        this.templateFile.presentationTemplates.appendChild(this.node);
      }
    }

    // build source of template
    fullSource  = '<template ';
    fullSource += 'operationName="' + this.operationName + '" ';
    fullSource += 'indexCode="' + this.indexCode + '"';
    if (!this.isContentTemplate) {
      fullSource += '\n      ';
      fullSource += 'xml:lang="' + this["xml:lang"] + '" ';
      fullSource += 'markupLanguage="' + this.markupLanguage + '" ';
      fullSource += 'alternateIndex="' + this.alternateIndex + '" ';
    }
    fullSource += ">\n";

    if (this.isBase) {
      fullSource += "      <basePrefix><![CDATA[" + this.basePrefix + "]]></basePrefix>\n";
      fullSource += "      <wrapperPrefix><![CDATA[" + this.wrapperPrefix + "]]></wrapperPrefix>\n";
    } else {
      fullSource += this.templateSource + "\n";
    }

    fullSource += "    </template>";

    // find insertion point and cut length for template
    if (this.isNewTemplate) {
      cutLength = 0; // we are not cutting any characters from the template, period
      if (this.isContentTemplate) {
        startIndex = this.templateFile.fileAsString.indexOf("</contentTemplates>");
      } else {
        startIndex = this.templateFile.fileAsString.indexOf("</presentationTemplates>");
      }
      fullSource = "\n" + fullSource + "\n  ";
      warn("We really should provide a sort mechanism for inserting new templates.");
    } else {
      startIndex = this.templateFile.fileAsString.indexOf(this.fileExcerpt.fullElement);
      cutLength = this.fileExcerpt.fullElement.length;
    }

    stringParts[0] = this.templateFile.fileAsString.substr(0, startIndex);
    stringParts[1] = fullSource;
    stringParts[2] = this.templateFile.fileAsString.substr(startIndex + cutLength);

    this.templateFile.fileAsString = stringParts.join("");
    this.templateFile.needsSave = true;
    this.templateEditCode = 0;
  },

  toString: function() {
    return "[object abacusTemplate]";
  }
}

function abacusTemplatesTable() {
  this.files = [];
  this.rows = [];
  this.userInterfaces = {};
}
abacusTemplatesTable.prototype = {
  addFile: function(aUrl, aTemplateFileCode) {
    var latestTemplateFile = new abacusTemplateFile(aUrl, aTemplateFileCode);
    this.files.push(latestTemplateFile);
    this.addTemplatesByNode(latestTemplateFile.contentTemplates, latestTemplateFile);
    this.addTemplatesByNode(latestTemplateFile.presentationTemplates, latestTemplateFile);
    this.addUIGroupsByFile(latestTemplateFile);
    return latestTemplateFile;
  },

  addTemplatesByNode: function(aTemplatesNode, aTemplateFile) {
    // no need to build a new TreeWalker just for this
    for (var k = 0; k < aTemplatesNode.childNodes.length; k++) {
      var aNode = aTemplatesNode.childNodes[k];
      var aTemplate = applyContractConstructor(abacusTemplate, aNode, aTemplateFile);
      this.rows.push(aTemplate);
      aTemplate.templateEditCode = 0; // per the specification in abacusTemplate();
    }
  },

  addTemplatesByNode_postcondition: function(aTemplatesNode, aTemplateFile, rv) {
    var errMsg = "applyContractConstructor did not correctly execute!"
    assert(this.rows[this.rows.length - 1] instanceof abacusTemplate, errMsg, true);
  },

  addUIGroupsByFile: function(aTemplateFile) {
    for (var i = 0; i < aTemplateFile.userInterfaces.childNodes.length; i++) {
      var uiGroupsNode = aTemplateFile.userInterfaces.childNodes[i];
      var humanLanguage = uiGroupsNode.getAttributeNS(XML_NS, "lang");
      var markupLanguage = uiGroupsNode.getAttribute("markupLanguage");
      assert(humanLanguage, aTemplateFile.url + " is missing a xml:lang attribute on a uiGroups element!", true);
      assert(markupLanguage, aTemplateFile.url + " is missing a markupLanguage attribute on a uiGroups element!", true);

      // set source code for uiGroupsNode
      uiGroupsNode.sourceObject = aTemplateFile.matchNodeToSource(uiGroupsNode, aTemplateFile.uiGroupsStrings,
                                                                  ["markupLanguage", "xml:lang"], false);

      // set label on menudeck
      var fileLeaf = aTemplateFile.fileObj.leaf;
      var menuListLabel = humanLanguage + "/" + markupLanguage + " (" + fileLeaf + ")";
      if (typeof this.userInterfaces[menuListLabel] == "undefined") {
        this.userInterfaces[menuListLabel] = [];
      }

      // add uiGroup elements for menubar menus
      for (var j = 0; j < uiGroupsNode.childNodes.length; j++) {
        var uiGroupNode = uiGroupsNode.childNodes[j];
        uiGroupNode.templateFile = aTemplateFile; // this comes in handy later when we need to save
        this.userInterfaces[menuListLabel].push(uiGroupNode);
      }
    }
  },

  dumpColumnValues: function(aColumnName) {
    var response = [];
    for (var i = 0; i < this.rows.length; i++) {
      if ((this.rows[i].templateFile.templateFileCode != abacusTemplateFile.BASE_FILE)||(ecmaDebug)) {
        response[i] = this.rows[i][aColumnName];
      }
    }
    response.sort();
    for (i = 0; i < response.length;) {
      if (response[i] == response[i + 1]) {
        response.splice(i, 1);
      } else {
        i++;
      }
    }
    return response;
  },

  meetsCriteria: function(searchCriteria, row) {
    var passesSearch = true;
    for (var j = 0; (passesSearch)&&(j < searchCriteria.length); j++) {
      var criterium = searchCriteria[j];
      passesSearch = (row[criterium.name].indexOf(criterium.value) != -1);
    }
    return passesSearch;
  },

  getFirstSearchResult: function(searchCriteria) {
    var rv = null;
    var row;
    var aRowSet;
    if (arguments.length > 1) {
      aRowSet = arguments[1];
    } else {
      aRowSet = this.rows;
    }
    for (var i = 0; i < aRowSet.length; i++) {
      row = aRowSet[i];
      if (this.meetsCriteria(searchCriteria, row)) {
        rv = row;
        break;
      }
    }
    return rv;
  },

  getMatchingTemplates: function(searchCriteria) {
    var rv = [];
    var aRowSet;
    if (arguments.length > 1) {
      aRowSet = arguments[1];
    } else {
      aRowSet = this.rows;
    }
    for (var i = 0; i < aRowSet.length; i++) {
      var row = aRowSet[i];
      if (this.meetsCriteria(searchCriteria, row)) {
        rv.push(row);
      }
    }
    return rv;
  },

  getTemplateFileByName: function(aFilename) {
    var i;
    var aTemplateFile;
    var chromeURL = "chrome://abacus/content/templates/" + aFilename;
    for (i = 0; i < this.files.length; i++) {
      aTemplateFile = this.files[i];
      if (aTemplateFile.url == chromeURL) {
        // Sweet.  We already loaded this file.
        return aTemplateFile;
      }
    }

    // Okay, we didn't load the file earlier.
    var urlPath = fUtils.chromeToPath(aFilename);
    var localPath = fUtils.urlToPath(urlPath);
    var fileObj = new File(localPath);
    if (!fileObj.exists()) {
      // The file we want to load isn't there.  We need to copy from the baseTemplate.xml file.
      var basePath = fUtils.chromeToPath("chrome://abacus/content/templates/baseTemplate.xml");
      var baseLocalPath = fUtils.urlToPath(basePath);
      var baseFile = new File(baseLocalPath);
      assert(baseFile.exists(), "baseTemplate.xml file could not be found!", true);
      baseFile.copy(localPath);
      assert(fileObj.exists(), "copy operation did not succeed! Where did the copied file land?", true);
    }

    aTemplateFile = this.addFile(aFilename, abacusTemplateFile.UNKNOWN_FILE);
    return aTemplateFile;
  },

  toString: function() {
    return "[object abacusTemplateTable]";
  }
}
setContract(abacusTemplatesTable.prototype.addTemplatesByNode,
            null,
            abacusTemplatesTable.prototype.addTemplatesByNode_postcondition);
